==> jira-projects-plugin-4.3.2.jar: page/project/abstract-list/SubmitModel-test.js <==
AJS.test.require(['com.atlassian.jira.jira-projects-plugin:abstract-list-page'], function () {
    "use strict";

    var SubmitModelStates = require("jira/projects/abstract-model/submit-model-states");
    var $ = require("jquery");

    module('SubmitModel', {
        setup: function () {
            this.context = AJS.test.mockableModuleContext();
            this.sandbox = sinon.sandbox.create();
            this.sandbox.useFakeServer();

            this.SubmitModel = this.context.require("jira/projects/abstract-list/submit-model");
            this.xhr = {};
            this.errorResponse = [400, {"Content-Type": "application/json"}, JSON.stringify({
                "errorMessages": [],
                "errors": {"name": "Some error"}
            })];

            this.successResponse = [200, {"Content-Type": "application/json"}, JSON.stringify({
                "some": "value",
                "other": 10
            })];

            this.detailsSyncDeferred = $.Deferred();

            this.model = new this.SubmitModel();
            this.model.toDataJSON = sinon.spy();
            this.detailsSyncStub = sinon.stub(this.model, 'detailsSync');
            this.detailsSyncStub.returns(this.detailsSyncDeferred);
            this.model.urlRoot = "/fakeUrl";
        },
        teardown: function () {
            this.sandbox.restore();
        }
    });

    test("save() should set state to SubmitModelStates.IN_PROGRESS", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.model.save();
        equal(this.model.get('state'), SubmitModelStates.IN_PROGRESS);
    });

    test("failed save() should set state to SubmitModelStates.ERROR_ADD", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.sandbox.server.respondWith(/.*\/fakeUrl.*/, this.errorResponse);
        this.model.save();
        this.sandbox.server.respond();
        equal(this.model.get('state'), SubmitModelStates.ERROR_ADD);
    });

    test("success save() should resolve deferred", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.sandbox.server.respondWith(/.*\/fakeUrl.*/, this.successResponse);
        var deferred = this.model.save();
        equal(deferred.isRejected(), false);
        equal(deferred.isResolved(), false);
        this.sandbox.server.respond();

        equal(deferred.isRejected(), false);
        equal(deferred.isResolved(), false);

        this.detailsSyncDeferred.resolve();

        equal(deferred.isRejected(), false);
        ok(deferred.isResolved());
    });

    test("failed save() should reject deferred", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.sandbox.server.respondWith(/.*\/fakeUrl.*/, this.errorResponse);
        var deferred = this.model.save();
        equal(deferred.isRejected(), false);
        equal(deferred.isResolved(), false);
        this.sandbox.server.respond();

        ok(deferred.isRejected());
        equal(deferred.isResolved(), false);
    });

    test("discard() on ERROR_ADD model should call destroy", function () {
        this.model.destroy = sinon.spy();

        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.set('state', SubmitModelStates.ERROR_ADD);
        equal(this.model.get('state'), SubmitModelStates.ERROR_ADD);

        sinon.assert.notCalled(this.model.destroy);
        this.model.discard();
        sinon.assert.calledOnce(this.model.destroy);
    });

    test("discard() on ERROR_DELETE model should set state to READY", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.set('state', SubmitModelStates.ERROR_DELETE);
        equal(this.model.get('state'), SubmitModelStates.ERROR_DELETE);
        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);
    })

    test("discard() on ERROR_UPDATE model should set state to READY", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.set('state', SubmitModelStates.ERROR_UPDATE);
        equal(this.model.get('state'), SubmitModelStates.ERROR_UPDATE);
        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);
    });
});

==> jira-projects-plugin-6.0.5.jar: page/project/abstract-list/SubmitModel-test.js <==
AJS.test.require(['com.atlassian.jira.jira-projects-plugin:abstract-list-page'], function () {
    "use strict";

    var SubmitModelStates = require("jira/projects/abstract-model/submit-model-states");
    var $ = require("jquery");

    var Deferred = require("jira/jquery/deferred");

    module('SubmitModel', {
        setup: function () {
            this.context = AJS.test.mockableModuleContext();
            this.sandbox = sinon.sandbox.create();
            this.sandbox.useFakeServer();

            this.SubmitModel = this.context.require("jira/projects/abstract-list/submit-model");
            this.xhr = {};
            this.errorResponse = [400, {"Content-Type": "application/json"}, JSON.stringify({
                "errorMessages": [],
                "errors": {"name": "Some error"}
            })];

            this.successResponse = [200, {"Content-Type": "application/json"}, JSON.stringify({
                "some": "value",
                "other": 10
            })];

            this.detailsSyncDeferred = new Deferred();

            this.model = new this.SubmitModel();
            this.model.toDataJSON = sinon.spy();
            this.detailsSyncStub = sinon.stub(this.model, 'detailsSync');
            this.detailsSyncStub.returns(this.detailsSyncDeferred);
            this.model.urlRoot = "/fakeUrl";
        },
        teardown: function () {
            this.sandbox.restore();
        }
    });

    test("save() should set state to SubmitModelStates.IN_PROGRESS", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.model.save();
        equal(this.model.get('state'), SubmitModelStates.IN_PROGRESS);
    });

    test("failed save() should set state to SubmitModelStates.ERROR_ADD", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.sandbox.server.respondWith(/.*\/fakeUrl.*/, this.errorResponse);
        this.model.save();
        this.sandbox.server.respond();
        equal(this.model.get('state'), SubmitModelStates.ERROR_ADD);
    });

    test("success save() should resolve deferred", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.sandbox.server.respondWith(/.*\/fakeUrl.*/, this.successResponse);
        var deferred = this.model.save();
        equal(deferred.state(), 'pending');
        this.sandbox.server.respond();

        equal(deferred.state(), 'pending');

        this.detailsSyncDeferred.resolve();

        equal(deferred.state(), 'resolved');
    });

    test("failed save() should reject deferred", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);
        this.sandbox.server.respondWith(/.*\/fakeUrl.*/, this.errorResponse);
        var deferred = this.model.save();
        equal(deferred.state(), 'pending');
        this.sandbox.server.respond();

        equal(deferred.state(), 'rejected');
    });

    test("discard() on ERROR_ADD model should call destroy", function () {
        this.model.destroy = sinon.spy();

        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.set('state', SubmitModelStates.ERROR_ADD);
        equal(this.model.get('state'), SubmitModelStates.ERROR_ADD);

        sinon.assert.notCalled(this.model.destroy);
        this.model.discard();
        sinon.assert.calledOnce(this.model.destroy);
    });

    test("discard() on ERROR_DELETE model should set state to READY", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.set('state', SubmitModelStates.ERROR_DELETE);
        equal(this.model.get('state'), SubmitModelStates.ERROR_DELETE);
        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);
    })

    test("discard() on ERROR_UPDATE model should set state to READY", function () {
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);

        this.model.set('state', SubmitModelStates.ERROR_UPDATE);
        equal(this.model.get('state'), SubmitModelStates.ERROR_UPDATE);
        this.model.discard();
        equal(this.model.get('state'), SubmitModelStates.READY);
    });
});
